/*
 *  Copyright (C) 2011-2014  OpenDungeons Team
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "entities/CraftedTrap.h"

#include "network/ODPacket.h"

#include "gamemap/GameMap.h"

#include "traps/TrapBoulder.h"
#include "traps/TrapCannon.h"
#include "traps/TrapSpike.h"

#include "utils/Random.h"
#include "utils/LogManager.h"

#include <iostream>

const int32_t NB_TURNS_OUTSIDE_HATCHERY_BEFORE_DIE = 30;
const int32_t NB_TURNS_DIE_BEFORE_REMOVE = 5;
const std::string EMPTY_STRING;

CraftedTrap::CraftedTrap(GameMap* gameMap, const std::string& forgeName, Trap::TrapType trapType) :
    RenderedMovableEntity(gameMap, forgeName, getMeshFromTrapType(trapType), 0.0f, false),
    mTrapType(trapType)
{
}

CraftedTrap::CraftedTrap(GameMap* gameMap) :
    RenderedMovableEntity(gameMap)
{
}

const std::string& CraftedTrap::getMeshFromTrapType(Trap::TrapType trapType)
{
    switch(trapType)
    {
        case Trap::TrapType::nullTrapType:
            return EMPTY_STRING;
        case Trap::TrapType::cannon:
            return TrapCannon::MESH_CANON;
        case Trap::TrapType::spike:
            return TrapSpike::MESH_SPIKE;
        case Trap::TrapType::boulder:
            return TrapBoulder::MESH_BOULDER;
    }
    OD_ASSERT_TRUE_MSG(false, "Wrong enum asked for CraftedTrap " + getName() + ", trapType="
        + Ogre::StringConverter::toString(static_cast<uint32_t>(trapType)));
    return EMPTY_STRING;
}

void CraftedTrap::setPosition(const Ogre::Vector3& v)
{
    Tile* oldTile = getPositionTile();
    RenderedMovableEntity::setPosition(v);
    Tile* tile = getPositionTile();
    OD_ASSERT_TRUE_MSG(tile != nullptr, "entityName=" + getName());
    if(tile == nullptr)
        return;
    if(tile == oldTile)
        return;

    if(oldTile != nullptr)
        oldTile->removeCraftedTrap(this);

    OD_ASSERT_TRUE(tile->addCraftedTrap(this));
}

void CraftedTrap::notifyEntityCarried(bool isCarried)
{
    Tile* myTile = getPositionTile();
    OD_ASSERT_TRUE_MSG(myTile != nullptr, "name=" + getName());
    if(myTile == nullptr)
        return;
    if(isCarried)
    {
        setIsOnMap(false);
        myTile->removeCraftedTrap(this);
    }
    else
    {
        setIsOnMap(true);
        myTile->addCraftedTrap(this);
    }
}

CraftedTrap* CraftedTrap::getCraftedTrapFromStream(GameMap* gameMap, std::istream& is)
{
    CraftedTrap* obj = new CraftedTrap(gameMap);
    return obj;
}

CraftedTrap* CraftedTrap::getCraftedTrapFromPacket(GameMap* gameMap, ODPacket& is)
{
    CraftedTrap* obj = new CraftedTrap(gameMap);
    return obj;
}

const char* CraftedTrap::getFormat()
{
    // TODO : implement saving/loading in the level file
    return "position";
}
